#include <opencv2/opencv.hpp>

int main(int argc, char ** argv) {

    cv::Mat image = cv::imread("../sheet-on-table.png", cv::IMREAD_COLOR);
    
    cv::imshow("original",image);
    cv::waitKey(0);

    cv::Mat gray;
    cv::cvtColor(image,gray,cv::COLOR_BGR2GRAY);

    cv::Mat binary;
    cv::threshold(gray,binary,0,255,cv::THRESH_BINARY | cv::THRESH_OTSU);

    cv::imshow("binary",binary);
    cv::waitKey(0);

    std::vector<std::vector<cv::Point>> contours;
    cv::findContours(binary, contours, cv::RETR_EXTERNAL, cv::CHAIN_APPROX_SIMPLE);
    cv::RotatedRect rotatedRect = cv::minAreaRect(contours[0]);
    printf("[%f, (%f,%f) %fx%f]\n",rotatedRect.angle,rotatedRect.center.x,rotatedRect.center.y,rotatedRect.size.width,rotatedRect.size.height);
    
    std::vector<cv::Point2f> points(4);
    rotatedRect.points(points.data());
    for (int i=0; i<int(points.size()); i++) {
        cv::line(binary, points[i], points[(i + 1) % int(points.size())], cv::Scalar(127), 2);
    }
    cv::imshow("binary",binary);
    cv::waitKey(0);

    double angle = rotatedRect.angle;
    cv::Mat warp_matrix = cv::getRotationMatrix2D(cv::Point(image.cols/2,image.rows/2),angle,1);
    cv::Mat rotated;
    cv::warpAffine(image,rotated,warp_matrix,image.size());
    cv::imshow("rotated",rotated);
    cv::waitKey(0);

    std::vector<cv::Point2f> rotatedPoints(4);
    cv::transform(points, rotatedPoints, warp_matrix);
    cv::Rect rect = cv::boundingRect(rotatedPoints);
    cv::Mat aligned = rotated(rect);

    cv::imshow("aligned",aligned);
    cv::waitKey(0);
    cv::destroyAllWindows();

    cv::imwrite("aligned.png",aligned);
    return 0;
}