#include <opencv2/opencv.hpp>

int main()
{
    cv::Mat whole = cv::imread("../iCub-whole-straight.png",cv::IMREAD_COLOR);
    cv::Mat part = cv::imread("../iCub-head.png",cv::IMREAD_COLOR);

    cv::Mat whole_gray, part_gray;
    cv::cvtColor(whole,whole_gray,cv::COLOR_BGR2GRAY);
    cv::cvtColor(part,part_gray,cv::COLOR_BGR2GRAY);
    
    cv::Mat whole_gray_f, part_gray_f;
    whole_gray.convertTo(whole_gray_f,CV_32F);
    part_gray.convertTo(part_gray_f,CV_32F);

    // phase correlation
    cv::Mat src1 = 1.0 - whole_gray_f/255.0;
    int h1 = whole_gray_f.rows, w1 = whole_gray_f.cols;
    cv::Mat src2(src1.rows,src1.cols,CV_32FC1,cv::Scalar(0));
    int h2 = part_gray_f.rows, w2 = part_gray_f.cols;
    src2(cv::Rect(0,0,w2,h2)) = 1.0 - part_gray_f/255.0;

    double confidence;
    cv::Point2d ret = cv::phaseCorrelate(src1,src2,cv::noArray(),&confidence);

    int x = -int(ret.x);
    if (x < 0) x += w1;

    int y = -int(ret.y);
    if (y < 0) y += h1;

    // phase correlation result: (x, y, w2, h2), confidence

    if (confidence > 0.1)
        cv::rectangle(whole,cv::Rect(x, y, w2, h2),cv::Scalar(0,0,255),2);

    cv::imshow("phase correlation", whole);
    cv::waitKey(0);
    cv::destroyAllWindows();

    return 0;
}
