#include <opencv2/opencv.hpp>

int main()
{
    cv::Mat whole = cv::imread("../iCub-whole-warped.png", cv::IMREAD_COLOR), whole_gray;
    cv::Mat part = cv::imread("../iCub-head.png", cv::IMREAD_COLOR), part_gray;

    cv::cvtColor(whole, whole_gray, cv::COLOR_BGR2GRAY);
    cv::cvtColor(part, part_gray, cv::COLOR_BGR2GRAY);

    cv::Size whole_sz = whole_gray.size();
    cv::Size part_sz = part_gray.size();

    // rough positioning of part to whole 
    cv::Point point(550, 275);

    std::vector<cv::Point2f> part_points{
        cv::Point2f(0,0), cv::Point2f(0,float(part_sz.height)),
        cv::Point2f(float(part_sz.width),float(part_sz.height)), cv::Point2f(float(part_sz.width),0)
    };
    std::vector<cv::Point2f> whole_points{
        cv::Point(point.x - part_sz.width / 2, point.y - part_sz.height / 2),
        cv::Point(point.x - part_sz.width / 2, point.y + part_sz.height / 2),
        cv::Point(point.x + part_sz.width / 2, point.y + part_sz.height / 2),
        cv::Point(point.x + part_sz.width / 2, point.y - part_sz.height / 2)
    };
    cv::Mat warp_matrix = cv::getAffineTransform(part_points.data(), whole_points.data());
    warp_matrix.convertTo(warp_matrix, CV_32F);

    // check only
    cv::Mat aligned_part;
    cv::warpAffine(part, aligned_part, warp_matrix, whole_sz, cv::INTER_LINEAR);
    std::vector<cv::Point2f> aligned_points_f(part_points.size());
    cv::transform(part_points, aligned_points_f, warp_matrix);
    // alignes_points_f == whole_points
    std::vector<cv::Point2i> aligned_points;
    for (cv::Point2f &point : aligned_points_f) aligned_points.push_back(cv::Point2i(point));
    cv::Mat aligned = whole.clone();
    for (int i = 0; i < 4; i++) cv::line(aligned, aligned_points[i], aligned_points[(i + 1) % 4], (0, 0, 255), 2);
    cv::imwrite("aligned0.png", aligned);
    cv::imshow("whole", whole);
    cv::imshow("part", part);
    cv::waitKey(0);
    cv::imshow("warped part", aligned_part);
    cv::imshow("aligned", aligned);
    cv::waitKey(0);

    try {
        // use ECC
        int warp_mode = cv::MOTION_AFFINE;
        int number_of_iterations = 200;
        double termination_eps = 1e-10;
        auto criteria = cv::TermCriteria(cv::TermCriteria::EPS + cv::TermCriteria::COUNT, number_of_iterations, termination_eps);
        cv::findTransformECC(part_gray, whole_gray, warp_matrix, warp_mode, criteria, cv::Mat(), 5);

        // display results
        cv::Mat aligned_part;
        cv::warpAffine(part, aligned_part, warp_matrix, whole_sz, cv::INTER_LINEAR);
        cv::transform(part_points, aligned_points_f, warp_matrix);
        aligned_points.clear();
        for (cv::Point2f &point : aligned_points_f) aligned_points.push_back(cv::Point2i(point));
        aligned = whole.clone();
        for (int i = 0; i < 4; i++) cv::line(aligned, aligned_points[i], aligned_points[(i + 1) % 4], (0, 0, 255), 2);

        cv::imwrite("aligned_part.png", aligned_part);
        cv::imwrite("aligned.png", aligned);
        cv::imshow("whole", whole);
        cv::imshow("part", part);
        cv::imshow("warped part", aligned_part);
        cv::imshow("aligned", aligned);
        cv::waitKey(0);
    }
    catch (const std::exception &ee) {
        std::cerr << "ECC diverged: " << ee.what() << std::endl;
    }

    cv::destroyAllWindows();
    return 0;
}

