import numpy as np
import cv2

whole = cv2.imread("../iCub-whole-warped.png",cv2.IMREAD_COLOR)
part = cv2.imread("../iCub-head.png",cv2.IMREAD_COLOR)

whole_gray = cv2.cvtColor(whole,cv2.COLOR_BGR2GRAY)
part_gray = cv2.cvtColor(part,cv2.COLOR_BGR2GRAY)
whole_sz = whole_gray.shape
part_sz = part_gray.shape

# rough positioning of part to whole 
point = (550,275)
part_points = np.array([ (0,0), (0,part_sz[0]), (part_sz[1],part_sz[0]), (part_sz[1],0) ],np.float32)
whole_points = np.array([ (point[0]-part_sz[1]//2,point[1]-part_sz[0]//2), (point[0]-part_sz[1]//2,point[1]+part_sz[0]//2), (point[0]+part_sz[1]//2,point[1]+part_sz[0]//2), (point[0]+part_sz[1]//2,point[1]-part_sz[0]//2) ],np.float32)
warp_matrix = np.asarray(cv2.getAffineTransform(part_points[:3],whole_points[:3]),np.float32)

# check only
aligned_part = cv2.warpAffine(part, warp_matrix, (whole_sz[1],whole_sz[0]), flags=cv2.INTER_LINEAR)
part_points_ext = np.append(part_points, np.ones((part_points.shape[0],1),np.float32), axis=1)
aligned_points = np.dot(part_points_ext, np.transpose(warp_matrix))
# alignes_points == whole_points
aligned_points = [ (int(point[0]),int(point[1])) for point in aligned_points ]
aligned = np.copy(whole)
for a, b in zip(range(4),np.roll(range(4),-1)):
    cv2.line(aligned,aligned_points[a],aligned_points[b],(0,0,255),2)
cv2.imwrite("aligned0.png",aligned)
cv2.imshow("whole", whole)
cv2.imshow("part", part)
cv2.waitKey(0)
cv2.imshow("warped part", aligned_part)
cv2.imshow("aligned", aligned)
cv2.waitKey(0)

try:
    # use ECC
    warp_mode = cv2.MOTION_AFFINE
    number_of_iterations = 200
    termination_eps = 1e-10
    criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, number_of_iterations,  termination_eps)
    (cc, warp_matrix) = cv2.findTransformECC(part_gray, whole_gray, warp_matrix, warp_mode, criteria, inputMask=None, gaussFiltSize=5)

    # display results
    aligned_part = cv2.warpAffine(part, warp_matrix, (whole_sz[1],whole_sz[0]), flags=cv2.INTER_LINEAR)
    part_points_ext = np.append(part_points, np.ones((part_points.shape[0],1),np.float32), axis=1)
    aligned_points = np.dot(part_points_ext, np.transpose(warp_matrix))
    aligned_points = [ (int(point[0]),int(point[1])) for point in aligned_points ]
    aligned = np.copy(whole)
    for a, b in zip(range(4),np.roll(range(4),-1)):
        cv2.line(aligned,aligned_points[a],aligned_points[b],(0,0,255),2)

    cv2.imwrite("aligned_part.png",aligned_part)
    cv2.imwrite("aligned.png",aligned)
    cv2.imshow("whole", whole)
    cv2.imshow("part", part)
    cv2.imshow("warped part", aligned_part)
    cv2.imshow("aligned", aligned)
    cv2.waitKey(0)

except Exception as ee:
    print('ECC diverged:', ee)
    
cv2.destroyAllWindows()

