import numpy as np
import cv2

part = cv2.imread("../iCub-head.png",cv2.IMREAD_COLOR)
whole = cv2.imread("../iCub-whole-warped.png",cv2.IMREAD_COLOR)

# create feature detector
featureDetector = cv2.SIFT_create()
#featureDetector = cv2.ORB_create(edgeThreshold=10, scoreType=cv2.ORB_HARRIS_SCORE, nfeatures=500)

# Detect target features and compute descriptors
keypoints_part, descriptors_part = featureDetector.detectAndCompute(part, None)

# Detect features and compute descriptors
keypoints_whole, descriptors_whole = featureDetector.detectAndCompute(whole, None)

# create descriptors matcher
matcher = cv2.DescriptorMatcher_create(cv2.DESCRIPTOR_MATCHER_BRUTEFORCE_L1)
#matcher = cv2.DescriptorMatcher_create(cv2.DESCRIPTOR_MATCHER_BRUTEFORCE_HAMMING)

# Match features
matches = matcher.match(descriptors_part, descriptors_whole, None)

# Sort matches by score
matches.sort(key=lambda x: x.distance, reverse=False)

# Remove not so good matches
numGoodMatches = int(len(matches) * 0.9)
matches = matches[:numGoodMatches]

# Draw top matches
imMatches = cv2.drawMatches(part, keypoints_part, whole, keypoints_whole, matches, None)
          
# Extract location of good matches
points_part = np.zeros((len(matches), 2), dtype=np.float32)
points_whole = np.zeros((len(matches), 2), dtype=np.float32)
for i, match in enumerate(matches):
    points_part[i, :] = keypoints_part[match.queryIdx].pt
    points_whole[i, :] = keypoints_whole[match.trainIdx].pt

# Find homography
homography, mask = cv2.findHomography(points_part, points_whole, cv2.RANSAC)

# Use homography
height_whole, width_whole = whole.shape[:2]
imRegistration = cv2.warpPerspective(part, homography, (width_whole, height_whole))

# Display results
disp_part = np.copy(part)
for p in points_part:
    cv2.circle(disp_part,tuple(p.astype(np.int)),1,(0,255,255),cv2.FILLED)
cv2.imwrite("keypoints-part.png",disp_part)
cv2.imshow('keypoints-part',disp_part)
cv2.waitKey(0)

disp_whole = np.copy(whole)
for p in points_whole:
    cv2.circle(disp_whole,tuple(p.astype(np.int)),1,(0,255,255),cv2.FILLED)
cv2.imwrite("keypoints-whole.png",disp_whole)
cv2.imshow('keypoints-part',disp_whole)
cv2.waitKey(0)

height_part, width_part = part.shape[:2]
points = np.array([[[0,0],[width_part-1,0],[width_part-1,height_part-1],[0,height_part-1]]],dtype=np.float32)
transformed_points = cv2.perspectiveTransform(points,homography)
x, y, w, h = cv2.boundingRect(transformed_points)
transformed_points = np.asarray(transformed_points[0]+(width_part,0),np.int)
for i in range(len(transformed_points)):
    cv2.line(imMatches,tuple(transformed_points[i]),tuple(transformed_points[(i+1)%len(transformed_points)]),(0,255,0),2)
cv2.rectangle(imMatches,(width_part+x,y),(width_part+x+w,y+h),(0,0,255),2) 
cv2.imwrite("matches.png", imMatches)
cv2.imshow('matches',imMatches)
cv2.waitKey(0)

cv2.imwrite('registration.png',imRegistration)
cv2.imshow('registration',imRegistration)
cv2.waitKey(0)

gray = cv2.cvtColor(cv2.cvtColor(whole,cv2.COLOR_BGR2GRAY),cv2.COLOR_GRAY2BGR)
mask = np.all(imRegistration != (0,0,0), axis=-1)
gray[mask] = (0,0,0)
imCombined = cv2.bitwise_or(gray,imRegistration)
cv2.imshow('combined',imCombined)
cv2.waitKey(0)

cv2.destroyAllWindows()
